@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Importer.cpp,v 1.16 2007/06/02 04:49:58 vkurland Exp $
+  $Id: Importer.cpp,v 1.17 2007/08/06 07:07:22 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -123,6 +123,16 @@ Importer::Importer(FWObject *_lib,
     current_interface = NULL;
     current_ruleset = NULL;
     current_rule = NULL;
+
+    tcp_flag_names[libfwbuilder::TCPService::URG]="u"; 
+    tcp_flag_names[libfwbuilder::TCPService::ACK]="a"; 
+    tcp_flag_names[libfwbuilder::TCPService::PSH]="p"; 
+    tcp_flag_names[libfwbuilder::TCPService::RST]="r"; 
+    tcp_flag_names[libfwbuilder::TCPService::SYN]="s"; 
+    tcp_flag_names[libfwbuilder::TCPService::FIN]="f"; 
+    tcp_flag_names[98]="N";   // NONE
+    tcp_flag_names[99]="A";   // ALL
+    
 }
 
 void Importer::run()
@@ -254,7 +264,14 @@ void Importer::addRuleComment(const std::string &comm)
     *logger << "Rule comment: " << comm << "\n";
 }
 
-UnidirectionalRuleSet* Importer::getUnidirRuleSet(const std::string &ruleset_name)
+UnidirectionalRuleSet* Importer::checkUnidirRuleSet(
+    const std::string &ruleset_name)
+{
+    return all_rulesets[ruleset_name];
+}
+
+UnidirectionalRuleSet* Importer::getUnidirRuleSet(
+    const std::string &ruleset_name)
 {
     UnidirectionalRuleSet *rs = all_rulesets[ruleset_name];
     if (rs==NULL)
@@ -547,10 +564,18 @@ FWObject* Importer::getTCPService(int srs, int sre,
     if (!flags_mask.empty() && !flags_comp.empty()) cstr << " flags";
 
     std::ostringstream nstr;
-    nstr << "tcp " 
-         << srs << "-" << sre << ":" << drs << "-" << dre;
+    nstr << "tcp " ;
+    if (srs!=0 || sre!=0)  nstr << srs << "-" << sre << ":";
+    if (drs!=0 || dre!=0)  nstr << drs << "-" << dre;
     if (established) nstr << " est";
-    if (!flags_mask.empty() && !flags_comp.empty()) nstr << " flags";
+    if (!flags_mask.empty() && !flags_comp.empty())
+    {
+        for (li = flags_mask.begin(); li != flags_mask.end(); ++li)
+            nstr << tcp_flag_names[*li];
+        nstr << "/";
+        for (li = flags_comp.begin(); li != flags_comp.end(); ++li)
+            nstr << tcp_flag_names[*li];
+    }
 
     TCPService* s = TCPService::cast(
         createObject(TCPService::TYPENAME, nstr.str()));
@@ -563,10 +588,38 @@ FWObject* Importer::getTCPService(int srs, int sre,
     {
         // TCP flags
         for (li = flags_mask.begin(); li != flags_mask.end(); ++li)
-            s->setTCPFlagMask( TCPService::TCPFlag(*li), true);
-
+        {
+            switch (*li)
+            {
+            case 99:    // ALL
+                s->setAllTCPFlagMasks();
+                break;
+            case 98:    // NONE
+                s->clearAllTCPFlagMasks();
+                break;
+            default:
+                s->setTCPFlagMask( TCPService::TCPFlag(*li), true);
+            }
+        }
         for (li = flags_comp.begin(); li != flags_comp.end(); ++li)
-            s->setTCPFlag( TCPService::TCPFlag(*li), true);
+        {
+            switch (*li)
+            {
+            case 99:    // ALL
+                 s->setTCPFlag( TCPService::URG, true);
+                 s->setTCPFlag( TCPService::ACK, true);
+                 s->setTCPFlag( TCPService::PSH, true);
+                 s->setTCPFlag( TCPService::RST, true);
+                 s->setTCPFlag( TCPService::SYN, true);
+                 s->setTCPFlag( TCPService::FIN, true);
+                break;
+            case 98:    // NONE
+                s->clearAllTCPFlags();
+                break;
+            default:
+                 s->setTCPFlag( TCPService::TCPFlag(*li), true);
+            }
+        }
     }
 
     s->setEstablished(established);
