@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: MangleTableCompiler_ipt.cpp,v 1.8 2007/01/05 06:47:05 vkurland Exp $
+  $Id: MangleTableCompiler_ipt.cpp,v 1.10 2007/07/08 01:11:34 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -66,25 +66,34 @@ bool MangleTableCompiler_ipt::keepMangleTableRules::processNext()
         ruleopt->getBool("ipt_branch_in_mangle"))
     {
         PolicyRule* r;
-
+        
         // this is a branching rule for mangle table. Need to put it
         // into PREROUTING and POSTROUTING chains as well because some
         // targets that work with mangle table can only go into these
         // chains, yet we do not know what kind of rules will user
         // place in the branch
 
-        r = PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
-        compiler->temp_ruleset->add(r);
-        r->duplicate(rule);
-        r->setStr("ipt_chain","PREROUTING");
-        tmp_queue.push_back(r);
+        if (rule->getDirection()==PolicyRule::Undefined ||
+            rule->getDirection()==PolicyRule::Both ||
+            rule->getDirection()==PolicyRule::Inbound)
+        {
+            r = PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            r->setStr("ipt_chain","PREROUTING");
+            tmp_queue.push_back(r);
+        }
 
-
-        r = PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
-        compiler->temp_ruleset->add(r);
-        r->duplicate(rule);
-        r->setStr("ipt_chain","POSTROUTING");
-        tmp_queue.push_back(r);
+        if (rule->getDirection()==PolicyRule::Undefined ||
+            rule->getDirection()==PolicyRule::Both ||
+            rule->getDirection()==PolicyRule::Outbound)
+        {
+            r = PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
+            compiler->temp_ruleset->add(r);
+            r->duplicate(rule);
+            r->setStr("ipt_chain","POSTROUTING");
+            tmp_queue.push_back(r);
+        }
 
         tmp_queue.push_back(rule);
     }
@@ -114,6 +123,12 @@ string MangleTableCompiler_ipt::flushAndSetDefaultPolicy()
         res << prp->_startRuleLine()
             << "PREROUTING -j CONNMARK --restore-mark"
             << prp->_endRuleLine();
+    }
+    if (have_connmark_in_output)
+    {
+        res << prp->_startRuleLine()
+            << "OUTPUT -j CONNMARK --restore-mark"
+            << prp->_endRuleLine();
         res << endl;
     }
 
