@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_PrintRule.cpp,v 1.42 2007/07/19 01:46:30 vkurland Exp $
+  $Id: PolicyCompiler_PrintRule.cpp,v 1.43 2007/12/13 16:39:09 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -1387,19 +1387,42 @@ string PolicyCompiler_ipt::PrintRule::_printOptionalGlobalRules()
                     << "FORWARD  -m state --state INVALID  -j drop_invalid"
                     << _endRuleLine();
 
+
+            res << _startRuleLine();
+
+
+	    if (compiler->getCachedFwOpt()->getBool("use_ULOG")) 
+	    {  
+	    	string s = compiler->getCachedFwOpt()->getStr("ulog_nlgroup");
+
+                res  << "drop_invalid -j ULOG ";
+
+        	if (!s.empty()) 
+            	    res << "--ulog-nlgroup " << s << " ";
+
+		int r;
+        	if ((r = compiler->getCachedFwOpt()->getInt("ulog_cprange"))!=0)  
+		     res << "--ulog-cprange " << r << " ";
+
+        	if ((r = compiler->getCachedFwOpt()->getInt("ulog_qthreshold"))!=0)  
+	  	     res << " --ulog-qthreshold " << r << " ";
+               
+	        res << "--ulog-prefix "; 
+
+	    } else {
+                   res << "drop_invalid -j LOG "
+		       << "--log-level debug --log-prefix ";
+            }
+
 	    string s = compiler->getCachedFwOpt()->getStr("log_prefix");
 	    if (s.empty())
-	      s = "INVALID state -- DENY ";
+	        s = "INVALID state -- DENY ";
 
-            res << _startRuleLine()
-                << "drop_invalid  -j LOG  --log-level debug --log-prefix "
-		<< _printLogPrefix("-1",
-                                   "DENY","global","drop_invalid","BLOCK INVALID",s)
-                << _endRuleLine();
-
-            res << _startRuleLine()
-                << "drop_invalid  -j DROP" << _endRuleLine();
-        }
+	    res << _printLogPrefix("-1", "DENY","global","drop_invalid","BLOCK INVALID",s)
+	        << _endRuleLine()
+                << _startRuleLine() << "drop_invalid  -j DROP" << _endRuleLine();
+ 
+	}
         res << endl;
     }
 
