@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw.cpp,v 1.12 2006/10/07 06:42:44 vkurland Exp $
+  $Id: PolicyCompiler_ipfw.cpp,v 1.13 2007/05/08 00:22:58 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -161,6 +161,7 @@ bool PolicyCompiler_ipfw::doSrcNegation::processNext()
     if (src->getNeg()) {
         RuleElementSrc *nsrc;
 	PolicyRule     *r;
+        FWOptions *ruleopt;
 
 	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
@@ -171,6 +172,8 @@ bool PolicyCompiler_ipfw::doSrcNegation::processNext()
         nsrc->setNeg(false);
 	r->setBool("quick",false);
         r->setBool("skip_check_for_duplicates",true);
+        ruleopt = r->getOptionsObject();
+        ruleopt->setBool("stateless", true);
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
@@ -199,6 +202,7 @@ bool PolicyCompiler_ipfw::doDstNegation::processNext()
     if (dst->getNeg()) {
         RuleElementDst *ndst;
 	PolicyRule     *r;
+        FWOptions *ruleopt;
 
 	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	compiler->temp_ruleset->add(r);
@@ -209,6 +213,8 @@ bool PolicyCompiler_ipfw::doDstNegation::processNext()
         ndst->setNeg(false);
 	r->setBool("quick",false);
         r->setBool("skip_check_for_duplicates",true);
+        ruleopt = r->getOptionsObject();
+        ruleopt->setBool("stateless", true);
 	tmp_queue.push_back(r);
 
 	r= PolicyRule::cast(compiler->dbcopy->create(PolicyRule::TYPENAME) );
