@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.13 2006/10/15 23:54:02 vkurland Exp $
+  $Id: PolicyCompiler_ipfw_writers.cpp,v 1.15 2007/05/08 02:11:39 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -128,43 +128,49 @@ string PolicyCompiler_ipfw::PrintRule::_printPort(int rs,int re,bool neg)
 string PolicyCompiler_ipfw::PrintRule::_printTCPFlags(TCPService *srv)
 {
     string str;
-    if (srv->inspectFlags())
+    if (srv->getEstablished())  str = "established";
+    else
     {
-        if (srv->getTCPFlagMask(TCPService::FIN))
+        if (srv->inspectFlags())
         {
-            if (!srv->getTCPFlag(TCPService::FIN)) str+="!";
-            str+="fin";
-        } 
-        if (srv->getTCPFlagMask(TCPService::SYN))
-        {
-            str+=",";
-            if (!srv->getTCPFlag(TCPService::SYN)) str+="!";
-            str+="syn";
-        } 
-        if (srv->getTCPFlagMask(TCPService::RST))
-        {
-            str+=",";
-            if (!srv->getTCPFlag(TCPService::RST)) str+="!";
-            str+="rst";
-        } 
-        if (srv->getTCPFlagMask(TCPService::PSH))
-        {
-            str+=",";
-            if (!srv->getTCPFlag(TCPService::PSH)) str+="!";
-            str+="psh";
-        } 
-        if (srv->getTCPFlagMask(TCPService::ACK))
-        {
-            str+=",";
-            if (!srv->getTCPFlag(TCPService::ACK)) str+="!";
-            str+="ack";
-        } 
-        if (srv->getTCPFlagMask(TCPService::URG))
-        {
-            str+=",";
-            if (!srv->getTCPFlag(TCPService::URG)) str+="!";
-            str+="urg";
-        } 
+            if (srv->getTCPFlagMask(TCPService::FIN))
+            {
+                if (!srv->getTCPFlag(TCPService::FIN)) str+="!";
+                str+="fin";
+            } 
+            if (srv->getTCPFlagMask(TCPService::SYN))
+            {
+                str+=",";
+                if (!srv->getTCPFlag(TCPService::SYN)) str+="!";
+                str+="syn";
+            } 
+            if (srv->getTCPFlagMask(TCPService::RST))
+            {
+                str+=",";
+                if (!srv->getTCPFlag(TCPService::RST)) str+="!";
+                str+="rst";
+            } 
+            if (srv->getTCPFlagMask(TCPService::PSH))
+            {
+                str+=",";
+                if (!srv->getTCPFlag(TCPService::PSH)) str+="!";
+                str+="psh";
+            } 
+            if (srv->getTCPFlagMask(TCPService::ACK))
+            {
+                str+=",";
+                if (!srv->getTCPFlag(TCPService::ACK)) str+="!";
+                str+="ack";
+            } 
+            if (srv->getTCPFlagMask(TCPService::URG))
+            {
+                str+=",";
+                if (!srv->getTCPFlag(TCPService::URG)) str+="!";
+                str+="urg";
+            } 
+            if (!str.empty())
+                return  "tcpflags " + str;
+        }
     }
     return str;
 }
@@ -425,7 +431,7 @@ void PolicyCompiler_ipfw::PrintRule::_printDstService(RuleElementSrv  *rel)
     if (TCPService::isA(s1)) 
     {
         string str=_printTCPFlags(TCPService::cast(s1));
-        if (!str.empty()) compiler->output << "tcpflags " << str << " ";
+        if (!str.empty()) compiler->output << str << " ";
     }
     if (IPService::isA(s1) && (s1->getBool("fragm") || s1->getBool("short_fragm")) )
         compiler->output << " frag ";
@@ -574,7 +580,7 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
     _printInterface(rule);
 
 /* keeping state does not apply to deny/reject */
-    if ( ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept) 
+    if ( ! ruleopt->getBool("stateless"))
     {  
 /*
  * this is per advice from Darren Reed  http://false.net/ipfilter/2002_12/0176.html
@@ -588,7 +594,9 @@ bool PolicyCompiler_ipfw::PrintRule::processNext()
  *
  */
         TCPService *tcpsrv=TCPService::cast(srv);
-        if ( tcpsrv!=NULL && !tcpsrv->inspectFlags() ) compiler->output << "setup ";
+        if ( tcpsrv!=NULL &&
+             !tcpsrv->inspectFlags() &&
+             !tcpsrv->getEstablished()  ) compiler->output << "setup ";
 
 	compiler->output << "keep-state ";
     }
